package gp;

import java.util.Collection;

/**
 * Created by IntelliJ IDEA.
 * User: julian
 * Date: Feb 8, 2007
 * Time: 3:30:58 PM
 */
public class Arithmetic implements Node {

    // not using an enum here because of wox
    static final int numberOfNodeTypes = 4;
    static final int plus = 0, minus = 1, mul = 2, div = 3;
    static String[] typeSymbols = new String[]{"+", "-", "*", "/"};

    private Node left;
    private Node right;
    private int type;
    private int maximumDepthAtCreation;
    private final int treeId;

    public Arithmetic(Node left, Node right, int type, int maximumDepth, int treeId) {
        this.treeId = treeId;
        this.left = left;
        this.right = right;
        this.type = type;
        this.maximumDepthAtCreation = maximumDepth;
    }

    public Arithmetic(Base base, Integer maximumDepth, Integer treeId) {
        this.treeId = treeId;
        left = base.newNode (maximumDepth - 1, treeId);
        right = base.newNode (maximumDepth - 1, treeId);
        type = (int) (Math.random () * numberOfNodeTypes);
        this.maximumDepthAtCreation = maximumDepth;
    }

    public void mutate(Base base) {
        if (Math.random () < 0.5) {
            left = base.newNode (maximumDepthAtCreation - 1, treeId);
        }
        else {
            right = base.newNode (maximumDepthAtCreation - 1, treeId);
        }
    }

    public Node copy() {
        return new Arithmetic (left.copy (), right.copy (), type, maximumDepthAtCreation, treeId);
    }

    public double value(Base base) {
        switch (type) {
            case plus:
                return left.value (base) + right.value (base);
            case minus:
                return left.value (base) - right.value (base);
            case mul:
                return left.value (base) * right.value (base);
            case div:
                double rval = right.value (base);
                if (rval != 0) {
                    return left.value (base) / rval;
                } else {
                    return 0;
                }
            default:
                System.out.println ("Arithmetic: unknown type:" + type);
                return 0;
        }
    }

    public int size() {
        return 1 + left.size () + right.size ();
    }

    public void listNodes(Collection<Node> collection) {
        collection.add (this);
        left.listNodes (collection);
        right.listNodes (collection);
    }

    public String toString () {
        return nodeString (left) + " " + typeSymbols[type] + " " + nodeString (right);
    }

    private String nodeString(Node node) {
        return node instanceof Terminal ? node.toString () : "(" + node.toString () + ")";
    }
}
