package gp;

import java.util.Collection;

/**
 * Created by IntelliJ IDEA.
 * User: julian
 * Date: Feb 8, 2007
 * Time: 4:08:58 PM
 */
public class IfThenElse implements Node {

    private Node ifNode;
    private Node thenNode;
    private Node elseNode;
    private int maximumDepthAtCreation;
    private final int treeId;

    public IfThenElse (Base base, Integer maximumDepth, Integer treeId) {
        this.treeId = treeId;
        ifNode = base.newNode (maximumDepth - 1, treeId);
        thenNode = base.newNode (maximumDepth - 1, treeId);
        elseNode = base.newNode (maximumDepth - 1, treeId);
        this.maximumDepthAtCreation = maximumDepth;
    }

    public IfThenElse (Node ifNode, Node thenNode, Node elseNode, int maximumDepth, int treeId) {
        this.treeId = treeId;
        this.ifNode = ifNode;
        this.thenNode = thenNode;
        this.elseNode = elseNode;
        this.maximumDepthAtCreation = maximumDepth;
    }

    public void mutate(Base base) {
        double selector = Math.random ();
        if (selector < 1/3.0) {
            ifNode = base.newNode (maximumDepthAtCreation - 1, treeId);
        }
        else if (selector < 2/3.0) {
            thenNode = base.newNode (maximumDepthAtCreation - 1, treeId);
        }
        else {
            elseNode = base.newNode (maximumDepthAtCreation - 1, treeId);
        }
    }

    public Node copy() {
        return new IfThenElse (ifNode.copy (), thenNode.copy (), elseNode.copy (), maximumDepthAtCreation, treeId);
    }

    public double value(Base base) {
        if (ifNode.value (base) > 0) {
            return thenNode.value (base);
        }
        else {
            return elseNode.value (base);
        }
    }

    public int size() {
        return 1 + ifNode.size () + thenNode.size () + elseNode.size ();
    }

    public void listNodes(Collection<Node> collection) {
        collection.add (this);
        ifNode.listNodes (collection);
        thenNode.listNodes (collection);
        elseNode.listNodes (collection);
    }

    public String toString () {
        return "if " + nodeString (ifNode) + " then " + nodeString (thenNode) + " else " + nodeString (elseNode);
    }

    private String nodeString (Node node) {
        return node instanceof Terminal ? node.toString () : "(" + node.toString () + ")";
    }
}
