package gp;

import java.util.Collection;

/**
 * Created by IntelliJ IDEA.
 * User: julian
 * Date: Feb 23, 2007
 * Time: 4:00:55 AM
 */
public class Trigonometric implements Node {

    // not using an enum here because of wox
    static final int numberOfNodeTypes = 4;
    static final int sin = 0, cos = 1, tan = 2, tanh = 3;
    static String[] typeSymbols = new String[]{"sin", "cos", "tan", "tanh"};

    private int type;
    private int maximumDepthAtCreation;
    private final int treeId;

    private Node child;

    public Trigonometric (Node child, int type, int maximumDepth, int treeId) {
        this.treeId = treeId;
        this.child = child;
        this.type = type;
        this.maximumDepthAtCreation = maximumDepth;
    }

    public Trigonometric (Base base, Integer maximumDepth, Integer treeId) {
        this.treeId = treeId;
        child = base.newNode (maximumDepth - 1, treeId);
        type = (int) (Math.random () * numberOfNodeTypes);
        this.maximumDepthAtCreation = maximumDepth;
    }

    public void mutate(Base base) {
        child = base.newNode (maximumDepthAtCreation - 1, treeId);
    }

    public Node copy() {
        return new Trigonometric (child.copy (), type, maximumDepthAtCreation, treeId);
    }

    public double value(Base base) {
        switch (type) {
            case 0:
                return Math.sin (child.value (base));
            case 1:
                return Math.cos (child.value (base));
            case 2:
                return Math.tan (child.value (base));
            case 3:
                return Math.tanh (child.value (base));
            default:
                System.out.println ("Trigonometric: unknown type:" + type);
                return 0;
        }
    }

    public int size() {
        return 1 + child.size ();
    }

    public void listNodes(Collection<Node> collection) {
        collection.add (this);
        child.listNodes (collection);
    }

    public String toString () {
        return typeSymbols[type] + " (" + child + ")";
    }
}
