package simplerace;

/**
 * Created by IntelliJ IDEA.
 * User: julian
 * Date: May 3, 2007
 * Time: 8:34:55 PM
 */
public class BasicSimulator implements Simulation {

    private Track track;
    private Vehicle firstVehicle;
    private Vehicle secondVehicle;

    public BasicSimulator (Track track, Vehicle firstVehicle, Vehicle secondVehicle) {
        this.setTrack (track);
        this.setFirstVehicle (firstVehicle);
        this.setSecondVehicle (secondVehicle);
    }

    public void setFirstVehicle(Vehicle vehicle) {
        track.setFirstVehicle (vehicle, false);
        this.firstVehicle = vehicle;
    }

    public void setSecondVehicle(Vehicle vehicle) {
        track.setSecondVehicle (vehicle, false);
        this.secondVehicle = vehicle;
    }

    public void setTrack(Track track) {
        this.track = track;
    }

    public Vehicle getFirstVehicle() {
        return firstVehicle;
    }

    public Vehicle getSecondVehicle() {
        return secondVehicle;
    }

    public Track getTrack() {
        return track;
    }

    public void update(int firstCommand, int secondCommand) {
        if (firstVehicle != null) {
            firstVehicle.update (firstCommand);
        }
        if (secondVehicle != null) {
            secondVehicle.update (secondCommand);
        }
        detectAndHandleCollision (firstVehicle, secondVehicle);
        track.update ();
    }

    public Simulation copy() {
            Vehicle firstVehicleCopy = null;
            if (firstVehicle != null) {
                firstVehicleCopy = firstVehicle.copy ();
            }
            Vehicle secondVehicleCopy = null;
            if (secondVehicle != null) {
                secondVehicleCopy = secondVehicle.copy ();
            }
            return new BasicSimulator (track.copy (), firstVehicleCopy, secondVehicleCopy);
    }

    private static void detectAndHandleCollision(Vehicle firstVehicle, Vehicle secondVehicle) {
        if (firstVehicle == null || secondVehicle == null) {
            return;
        }
        // First fix the car-to-car collisions
        Vector2d pointOfCollision = null;
        Vector2d car0col = firstVehicle.collisionWith (secondVehicle);
        Vector2d car1col = secondVehicle.collisionWith (firstVehicle);
        // Set the point of collision to the mean of what the cars report,
        // or to the only reported point if only one car reports a collision
        if (car0col != null) {
            pointOfCollision = new Vector2d (car0col.x, car0col.y);
        }
        if (car1col != null) {
            if (pointOfCollision == null) {
                pointOfCollision = new Vector2d (car1col.x, car1col.y);
            } else {
                pointOfCollision.add (car1col);
                pointOfCollision.div (2);
            }
        }
        // Tell both cars about the impact
        // Make sure that we use the "old" velocity values - that is, the ones
        // that we have before at the beginning of this time-step,
        // before the impact method is called on any vehicle
        if (pointOfCollision != null) {
            // got to make copies of velocities before they change
            // for obvious reasons (when you think about it)
            Vector2d firstVelocity = firstVehicle.getVelocity ();
            Vector2d secondVelocity = secondVehicle.getVelocity ();
            firstVehicle.impact (pointOfCollision, secondVelocity, 1);
            secondVehicle.impact (pointOfCollision, firstVelocity, 1);
        }
    }

}
