package simplerace;

import java.util.StringTokenizer;
import java.net.Socket;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.PrintStream;

/**
 * Created by IntelliJ IDEA.
 * User: julian
 * Date: Feb 12, 2007
 * Time: 12:27:26 AM
 */
public class Client implements Constants {

    static private BufferedReader reader;
    static private PrintStream printer;
    static private Socket socket;
    static private Controller controller;

    public static void main(String[] args) throws Exception {
        controller = Play.load (args[0]);
        int portToUse = tcpPortOne;
        if (args.length > 1 && !args[1].equals ("1")) {
            portToUse = tcpPortTwo;
        }
        System.out.println ("Trying to connect to server at port " + portToUse + "...");
        socket = new Socket ("localhost", portToUse);
        System.out.println ("Connected to " + socket.getInetAddress ());
        reader = new BufferedReader (new InputStreamReader (socket.getInputStream ()));
        printer = new PrintStream (socket.getOutputStream ());
        loop ();
    }

    private static void loop() {
        FakeSensorModel sensors = new FakeSensorModel ();
        while (socket.isConnected ()) {
            try {
                String data = reader.readLine ();
                if (data.startsWith ("reset")) {
                    controller.reset ();
                } else {
                    sensors.parse (data);
                }
                int action = controller.control (sensors);
                printer.println (action);
            }
            catch (Exception e) {
                e.printStackTrace ();
                System.exit (0);
            }
        }
    }

    public static class FakeSensorModel implements SensorModel {

        private double speed;
        private double angleToNextWaypoint;
        private double distanceToNextWaypoint;
        private double angleToNextNextWaypoint;
        private double distanceToNextNextWaypoint;
        private double angleToOtherVehicle;
        private double distanceToOtherVehicle;
        private boolean otherVehicleIsPresent;
        private Vector2d position = new Vector2d ();
        private Vector2d velocity = new Vector2d ();
        private double orientation;
        private Vector2d otherPosition = new Vector2d ();
        private Vector2d otherVelocity = new Vector2d ();
        private double otherOrientation;
        private Vector2d nextWaypoint = new Vector2d ();
        private Vector2d nextNextWaypoint = new Vector2d ();

        public void parse(String data) {
            StringTokenizer st = new StringTokenizer (data);
            String type = st.nextToken ();
            // first person data
            otherVehicleIsPresent = Boolean.parseBoolean (st.nextToken ());
            speed = Double.parseDouble (st.nextToken ());
            angleToNextWaypoint = Double.parseDouble (st.nextToken ());
            distanceToNextWaypoint = Double.parseDouble (st.nextToken ());
            angleToNextNextWaypoint = Double.parseDouble (st.nextToken ());
            distanceToNextNextWaypoint = Double.parseDouble (st.nextToken ());
            angleToOtherVehicle = Double.parseDouble (st.nextToken ());
            distanceToOtherVehicle = Double.parseDouble (st.nextToken ());
            // third person data
            position.x = Double.parseDouble (st.nextToken ());
            position.y = Double.parseDouble (st.nextToken ());
            velocity.x = Double.parseDouble (st.nextToken ());
            velocity.y = Double.parseDouble (st.nextToken ());
            orientation = Double.parseDouble (st.nextToken ());
            otherPosition.x = Double.parseDouble (st.nextToken ());
            otherPosition.y = Double.parseDouble (st.nextToken ());
            otherVelocity.x = Double.parseDouble (st.nextToken ());
            otherVelocity.y = Double.parseDouble (st.nextToken ());
            otherOrientation = Double.parseDouble (st.nextToken ());
            nextWaypoint.x = Double.parseDouble (st.nextToken ());
            nextWaypoint.y = Double.parseDouble (st.nextToken ());
            nextNextWaypoint.x = Double.parseDouble (st.nextToken ());
            nextNextWaypoint.y = Double.parseDouble (st.nextToken ());
        }

        public double getSpeed() {
            return speed;
        }

        public double getAngleToNextWaypoint() {
            return angleToNextWaypoint;
        }

        public double getDistanceToNextWaypoint() {
            return distanceToNextWaypoint;
        }

        public double getAngleToNextNextWaypoint() {
            return angleToNextNextWaypoint;
        }

        public double getDistanceToNextNextWaypoint() {
            return distanceToNextNextWaypoint;
        }

        public double getAngleToOtherVehicle() {
            return angleToOtherVehicle;
        }

        public double getDistanceToOtherVehicle() {
            return distanceToOtherVehicle;
        }

        public boolean otherVehicleIsPresent() {
            return otherVehicleIsPresent;
        }

        public boolean justPassedAWaypoint() {
            return false;  //todo fix
        }

        public boolean otherVehicleJustPassedAWaypoint() {
            return false;  //todo fix
        }

        public boolean justCollidedWithOtherVehicle() {
            return false;  //todo fix
        }

        public Vector2d getPosition() {
            return position;
        }

        public Vector2d getVelocity() {
            return velocity;
        }

        public double getOrientation() {
            return orientation;
        }

        public double getAngularVelocity() {
            return 0;  //TODO fix
        }

        public Vector2d getOtherVehiclePosition() {
            return otherPosition;
        }

        public Vector2d getOtherVehicleVelocity() {
            return otherVelocity;
        }

        public double getOtherVehicleOrientation() {
            return otherOrientation;
        }

        public double getOtherVehicleAngularVelocity() {
            return 0;  //TODO fix
        }

        public Vector2d getNextWaypointPosition() {
            return nextWaypoint;
        }

        public Vector2d getNextNextWaypointPosition() {
            return nextNextWaypoint;
        }

        public Simulation getSimulation() {
            return null;  //todo fix
        }
    }


}
