package simplerace;

import wox.serial.Easy;

/**
 * Created by IntelliJ IDEA.
 * User: jtogel
 * Date: 24-Oct-2006
 * Time: 02:55:01
 * To change this template use File | Settings | File Templates.
 */
public class CoEvolveOnePop {

    static final int popsize = 30;
    static final int elite = popsize / 2;
    static final int evaluationRepetitions = 5;
    static Evolvable[] population = new Evolvable[popsize];
    static int generations = 200;
    static double[] fitness = new double[popsize];
    static int[] numberOfTrials = new int[popsize];

    public static void main (String[] args) {
        Evolvable initialController;
        if (args.length > 0) {
            initialController = (Evolvable) Play.load (args[0]);
        }
        else {
            initialController = new MLPController ();
        }
        if (args.length > 1) {
            generations = Integer.parseInt (args[1]);
        }
        for (int i = 0; i < population.length; i++) {
            population[i] = initialController.copy ();
        }
        double lastBestFitness = 0;
        for (int i = 0; i < generations; i++) {
            oneMoreGeneration ();
            if (fitness[0] > lastBestFitness) {
                Easy.save (population[0], "coevolved.xml");
                lastBestFitness = fitness[0];
            }
            System.out.printf ("Gen %d fitness %.4f\n", i, fitness[0]);
        }
    }

    public static void oneMoreGeneration () {
        for (int i = 0; i < population.length; i++) {
            fitness[i] = 0;
            numberOfTrials[i] = 0;
        }
        for (int i = 0; i < elite; i++) {
            evaluate (i);
        }
        for (int i = elite; i < population.length; i++) {
            population[i] = population[i - elite].copy ();
            population[i].mutate ();
            evaluate (i);
        }
        averageFitnesses ();
        sortPopulationByFitness ();
    }

    private static void evaluate (int which) {
        for (int i = 0; i < evaluationRepetitions; i++) {
            int other = (int) (Math.random () * elite);
            population[which].reset ();
            population[other].reset ();
            double[] results = Evaluator.evaluateDuo ((Controller) population[which],
                    (Controller) population[other], new BasicTrack (), false);
            fitness[which] += results[0];
            numberOfTrials[which]++;
            fitness[other] += results[1];
            numberOfTrials[other]++;
        }
    }

    private static void averageFitnesses () {
        for (int i = 0; i < fitness.length; i++) {
            fitness[i] = fitness[i] / numberOfTrials[i];
        }
    }

    private static void sortPopulationByFitness () {
        for (int i = 0; i < population.length; i++) {
            for (int j = i + 1; j < population.length; j++) {
                if (fitness[i] < fitness[j]) {
                    double cache = fitness[i];
                    fitness[i] = fitness[j];
                    fitness[j] = cache;
                    Evolvable gcache = population[i];
                    population[i] = population[j];
                    population[j] = gcache;
                }
            }
        }
    }

}
