package simplerace;

import javax.swing.*;
import java.awt.event.KeyAdapter;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

/**
 * Created by IntelliJ IDEA.
 * User: julian
 * Date: 08-May-2007
 * Time: 12:28:45
 */
public class Evaluator implements Constants {


    public static double evaluateSolo (Controller controller, Track track, boolean visual) {
        Vehicle vehicle = new Car ();
        track.reset ();
        track.setFirstVehicle (vehicle, true);
        SensorModel sensors = track.getFirstSensorModel ();
        View view = null;
        Simulation simulation = new BasicSimulator (track, vehicle, null);
        if (visual) {
            view = new View (vehicle, null, track);
            initializeVisual (view,
                    controller instanceof KeyAdapter ? (KeyAdapter)controller : null,
                    null);
        }
        for (int i = 0; i < 1000; i++) {
            int command = controller.control (sensors);
            simulation.update (command, neutral);
            if (visual) {
                view.repaint ();
                try {
                    Thread.sleep (50);
                }
                catch (Exception e) {
                    e.printStackTrace ();
                }
            }
        }
        return track.getFirstFitness ();
    }


    public static double[] evaluateDuo (Controller firstController,
                                            Controller secondController, Track track, boolean visual) {
        Vehicle firstVehicle = new Car ();
        Vehicle secondVehicle = new Car ();
        track.reset ();
        track.setFirstVehicle (firstVehicle, true);
        track.setSecondVehicle (secondVehicle, true);
        SensorModel firstSensors = track.getFirstSensorModel ();
        SensorModel secondSensors = track.getSecondSensorModel ();
        View view = null;
        Simulation simulation = new BasicSimulator (track, firstVehicle, secondVehicle);
        if (visual) {
            view = new View (firstVehicle, secondVehicle, track);
            initializeVisual (view,
                    firstController instanceof KeyAdapter ? (KeyAdapter) firstController : null,
                    secondController instanceof KeyAdapter ? (KeyAdapter) secondController : null);
        }
        for (int i = 0; i < 1000; i++) {
            int firstCommand = firstController.control (firstSensors);
            int secondCommand = secondController.control (secondSensors);
            simulation.update (firstCommand, secondCommand);
            if (visual) {
                view.repaint ();
                try {
                    Thread.sleep (50);
                }
                catch (Exception e) {
                    e.printStackTrace ();
                }
            }
        }
        return new double[]{track.getFirstFitness (), track.getSecondFitness ()};
    }

    public static void initializeVisual (View view, KeyAdapter adapter1, KeyAdapter adapter2) {
        JFrame frame = new JFrame (" ");
        if (adapter1 != null) {
            frame.addKeyListener (adapter1);
        }
        if (adapter2 != null) {
            frame.addKeyListener (adapter2);
        }
        frame.addWindowListener (new WindowAdapter() {
            public void windowClosing (WindowEvent e) {
                System.exit (0);
            }
        });
        frame.getContentPane ().add (view);
        frame.pack ();
        frame.setVisible (true);
        view.setFrame (frame);
    }
}
