package simplerace;

import wox.serial.Easy;

/**
 * Created by IntelliJ IDEA.
 * User: julian
 * Date: Mar 21, 2007
 * Time: 1:27:53 AM
 */
public class EvolveAgainstFixed {

    static final int popsize = 30;
    static final int elite = popsize / 2;
    static final int evaluationRepetitions = 5;
    static Evolvable[] population = new Evolvable[popsize];
    static int generations = 200;
    static double[] fitness = new double[popsize];
    static Controller fixedCompetitor;

    public static void main(String[] args) {
        Evolvable initialController = (Evolvable) Play.load (args[0]);
        fixedCompetitor = (Controller) Play.load (args[1]);
        System.out.println ("Evaluating against " + fixedCompetitor);
        if (args.length > 2) {
            generations = Integer.parseInt (args[2]);
        }
        for (int i = 0; i < population.length; i++) {
            population[i] = initialController.copy ();
        }
        double lastBestFitness = 0;
        for (int i = 0; i < generations; i++) {
            oneMoreGeneration ();
            if (fitness[0] > lastBestFitness) {
                Easy.save (population[0], "evolved.xml");
                lastBestFitness = fitness[0];
            }
            System.out.printf ("Gen %d fitness %.4f\n", i, fitness[0]);
        }
    }

    public static void oneMoreGeneration() {
        for (int i = 0; i < elite; i++) {
            evaluate (i);
        }
        for (int i = elite; i < population.length; i++) {
            population[i] = population[i - elite].copy ();
            population[i].mutate ();
            evaluate (i);
        }
        sortPopulationByFitness ();
    }

    private static void evaluate(int which) {
        fitness[which] = 0;
        for (int i = 0; i < evaluationRepetitions; i++) {
            population[which].reset ();
            fitness[which] += Evaluator.evaluateDuo ((Controller) population[which], fixedCompetitor,
                    new BasicTrack (), false)[0];
        }
        fitness[which] = fitness[which] / evaluationRepetitions;
    }

    private static void sortPopulationByFitness() {
        for (int i = 0; i < population.length; i++) {
            for (int j = i + 1; j < population.length; j++) {
                if (fitness[i] < fitness[j]) {
                    double cache = fitness[i];
                    fitness[i] = fitness[j];
                    fitness[j] = cache;
                    Evolvable gcache = population[i];
                    population[i] = population[j];
                    population[j] = gcache;
                }
            }
        }
    }

}
