package simplerace;

import javax.swing.*;
import java.awt.event.KeyAdapter;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

/**
 * Created by IntelliJ IDEA.
 * User: jtogel
 * Date: 23-Oct-2006
 * Time: 17:13:31
 */
public class OldEvaluator {

    public static double evaluateSolo (Controller controller, Track track, boolean visual) {
        Vehicle vehicle = new Car ();
        track.reset ();
        track.setFirstVehicle (vehicle, true);
        SensorModel sensors = track.getFirstSensorModel ();
        View view = null;
        if (visual) {
            view = new View (vehicle, null, track);
            initializeVisual (view,
                    controller instanceof KeyAdapter ? (KeyAdapter)controller : null,
                    null);
        }
        for (int i = 0; i < 1000; i++) {
            int command = controller.control (sensors);
            vehicle.update (command);
            track.update ();
            if (visual) {
                view.repaint ();
                try {
                    Thread.sleep (50);
                }
                catch (Exception e) {
                    e.printStackTrace ();
                }
            }
        }
        return track.getFirstFitness ();
    }

    public static double[] evaluateDuo (Controller firstController, Controller secondController,
                                        Track track, boolean visual) {
        Vehicle firstVehicle = new Car ();
        Vehicle secondVehicle = new Car ();
        track.reset ();
        track.setFirstVehicle (firstVehicle, true);
        track.setSecondVehicle (secondVehicle, true);
        SensorModel firstSensors = track.getFirstSensorModel ();
        SensorModel secondSensors = track.getSecondSensorModel ();
        View view = null;
        if (visual) {
            view = new View (firstVehicle, secondVehicle, track);
            initializeVisual (view,
                    firstController instanceof KeyAdapter ? (KeyAdapter) firstController : null,
                    secondController instanceof KeyAdapter ? (KeyAdapter) secondController : null);
        }
        for (int i = 0; i < 1000; i++) {
            detectAndHandleCollision (firstVehicle, secondVehicle);
            int firstCommand = firstController.control (firstSensors);
            firstVehicle.update (firstCommand);
            int secondCommand = secondController.control (secondSensors);
            secondVehicle.update (secondCommand);
            track.update ();
            if (visual) {
                view.repaint ();
                try {
                    Thread.sleep (50);
                }
                catch (Exception e) {
                    e.printStackTrace ();
                }
            }
        }
        return new double[]{track.getFirstFitness (), track.getSecondFitness ()};
    }

    private static void initializeVisual (View view, KeyAdapter adapter1, KeyAdapter adapter2) {
        JFrame frame = new JFrame (" ");
        if (adapter1 != null) {
            frame.addKeyListener (adapter1);
        }
        if (adapter2 != null) {
            frame.addKeyListener (adapter2);
        }
        frame.addWindowListener (new WindowAdapter() {
            public void windowClosing (WindowEvent e) {
                System.exit (0);
            }
        });
        frame.getContentPane ().add (view);
        frame.pack ();
        frame.setVisible (true);
        view.setFrame (frame);
    }

    private static void detectAndHandleCollision (Vehicle firstVehicle, Vehicle secondVehicle) {
        // First fix the car-to-car collisions
        Vector2d pointOfCollision = null;
        Vector2d car0col = firstVehicle.collisionWith (secondVehicle);
        Vector2d car1col = secondVehicle.collisionWith (firstVehicle);
        // Set the point of collision to the mean of what the cars report,
        // or to the only reported point if only one car reports a collision
        if (car0col != null) {
            pointOfCollision = new Vector2d (car0col.x, car0col.y);
        }
        if (car1col != null) {
            if (pointOfCollision == null) {
                pointOfCollision = new Vector2d (car1col.x, car1col.y);
            } else {
                pointOfCollision.add (car1col);
                pointOfCollision.div (2);
            }
        }
        // Tell both cars about the impact
        // Make sure that we use the "old" velocity values - that is, the ones
        // that we have before at the beginning of this time-step,
        // before the impact method is called on any vehicle
        if (pointOfCollision != null) {
            // got to make copies of velocities before they change
            // for obvious reasons (when you think about it)
            Vector2d firstVelocity = firstVehicle.getVelocity ();
            Vector2d secondVelocity = secondVehicle.getVelocity ();
            firstVehicle.impact (pointOfCollision, secondVelocity, 1);
            secondVehicle.impact (pointOfCollision, firstVelocity, 1);
        }
    }

}
