package simplerace;

/**
 * Created by IntelliJ IDEA.
 * User: julian
 * Date: Jun 28, 2007
 * Time: 2:59:38 AM
 */
public class StateBasedPerceptronController implements Controller, Evolvable, Reinforcable, Constants {

    final int numberOfUpdatesPerAction = 2;
    private Perceptron perceptron;
    private boolean weAreTDLearning = false;
    private double totalReinforcement = 0;
    //private int lastBestAction = neutral;
    private double[] lastState = new double[4];
    private final double learningRate = 0.05;
    private final double discountRate = 0.2;
    private final double probabilityOfRandomActionWhenTDLearning = 0.1;

    public StateBasedPerceptronController() {
        this.perceptron = new Perceptron (4, 1);
    }

    private StateBasedPerceptronController(Perceptron perceptron) {
        this.perceptron = perceptron;
    }

    public void mutate() {
        perceptron.mutate ();
    }

    public Evolvable copy() {
        return new StateBasedPerceptronController (perceptron.copy ());
    }

    public void reset() {
    }

    // TODO this works great if we are the red car, but how do we know which car we are?

    public int control(SensorModel inputs) {
        if (weAreTDLearning) {
            learnFromReinforcements (inputs);
            totalReinforcement = 0;
        }
        double[] stateValues = new double[numberOfActions];
        // get values of the states resulting from taking each action
        for (int action = 0; action < numberOfActions; action++) {
            Simulation simulation = inputs.getSimulation ();
            for (int update = 0; update < numberOfUpdatesPerAction; update++) {
                simulation.update (action, neutral);
            }
            SensorModel virtualInputs = simulation.getTrack ().getFirstSensorModel ();
            double stateValue = perceptron.propagate (calculateStateVector (virtualInputs))[0];
            stateValues[action] = stateValue;
        }
        // select the action yielding the highest-value state
        int bestAction = 0;
        double bestValue = Double.NEGATIVE_INFINITY;
        for (int i = 0; i < numberOfActions; i++) {
            if (stateValues[i] > bestValue) {
                bestAction = i;
                bestValue = stateValues[i];
            }
        }
        // save state for learning
        if (weAreTDLearning) {
            //lastBestAction = bestAction;
            lastState = calculateStateVector (inputs);
            // with small probability, take a random action
            if (Math.random () < probabilityOfRandomActionWhenTDLearning) {
                return randomAction ();
            }
        }
        // return it
        return bestAction;
    }

    private void learnFromReinforcements(SensorModel inputs) {
        double[] oldState = lastState; // s
        double reward = totalReinforcement; // r
        double[] newState = calculateStateVector (inputs); // s'
        // TD(0): V(s) = V(s) + alpha * (r + gamma * V(s') - V(s));

        double newStateValue = perceptron.propagate (newState)[0]; // V(s')
        double oldStateValue = perceptron.propagate (oldState)[0]; // V(s)

        double targetValue = oldStateValue + learningRate * (reward + discountRate * newStateValue - oldStateValue);
        perceptron.learn (new double[]{targetValue});

    }

    public double[] calculateStateVector(SensorModel inputs) {
        return new double[]{1, inputs.getAngleToNextWaypoint (),
                inputs.getDistanceToNextWaypoint (), inputs.getSpeed ()};
        /*return new double[]{1, inputs.getAngleToNextWaypoint (),
                 inputs.getSpeed ()};*/
    }

    public void reinforce(double reinforcement) {
        weAreTDLearning = true;
        totalReinforcement = totalReinforcement + reinforcement;
    }

    private int randomAction() {
        return (int) (Math.random () * 9);
    }
}