package simplerace;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.PrintStream;
import java.net.Socket;
import java.net.ServerSocket;

/**
 * Created by IntelliJ IDEA.
 * User: julian
 * Date: Feb 11, 2007
 * Time: 9:36:59 PM
 */
public abstract class TCPController implements Controller, Runnable, Constants {

    private BufferedReader reader;
    private PrintStream printer;
    private Socket socket;
    private boolean go = false;
    private SensorModel inputs = null;
    private int action = neutral;

    public TCPController () {
        // This class is abstract, so this constructor is never used
    }

    public TCPController (int portToUse) {
        try {
            ServerSocket ss = new ServerSocket (portToUse);
            System.out.println ("Waiting for a client to connect on port " + portToUse);
            socket = ss.accept ();
            System.out.println ("We have a connection from " + socket.getInetAddress ());
            reader = new BufferedReader (new InputStreamReader (socket.getInputStream ()));
            printer = new PrintStream (socket.getOutputStream ());
            printer.println ("reset");
            Thread thread = new Thread (this);
            thread.start ();
        }
        catch (Exception e) {
            e.printStackTrace ();
        }
    }

    public void reset() {
    }

    public int control(SensorModel inputs) {
        this.inputs = inputs;
        go = true;
        int count = 0;
        while (go) {
            if (count > 50) {
                crash ("No reply received from client within 50 ms");
            }
            try {
                Thread.sleep (1);
            }
            catch (Exception e) {
                e.printStackTrace ();
            }
            count++;
        }
        return action;
    }

    public void run() {
        while (socket.isConnected ()) {
            try {
                while (!go) {
                    // wait for go
                    try {
                        Thread.sleep (1);
                    }
                    catch (Exception e) {
                        e.printStackTrace ();
                    }
                }
                // go!
                String firstPersonData = "data " +
                        // first person data
                        inputs.otherVehicleIsPresent () + " " +
                        inputs.getSpeed () + " " +
                        inputs.getAngleToNextWaypoint () + " " +
                        inputs.getDistanceToNextWaypoint () + " " +
                        inputs.getAngleToNextNextWaypoint () + " " +
                        inputs.getDistanceToNextNextWaypoint () + " " +
                        inputs.getAngleToOtherVehicle () + " " +
                        inputs.getDistanceToOtherVehicle () + " " +
                        // third person data
                        inputs.getPosition ().x + " " +
                        inputs.getPosition ().y + " " +
                        inputs.getVelocity ().x + " " +
                        inputs.getVelocity ().y + " " +
                        inputs.getOrientation () + " " +
                        inputs.getOtherVehiclePosition ().x + " " +
                        inputs.getOtherVehiclePosition ().y + " " +
                        inputs.getOtherVehicleVelocity ().x + " " +
                        inputs.getOtherVehicleVelocity ().y + " " +
                        inputs.getOtherVehicleOrientation () + " " +
                        inputs.getNextWaypointPosition ().x + " " +
                        inputs.getNextWaypointPosition ().y + " " +
                        inputs.getNextNextWaypointPosition ().x + " " +
                        inputs.getNextNextWaypointPosition ().y + " ";
                printer.println (firstPersonData);
                String commandString = reader.readLine ();
                action = Integer.parseInt (commandString);
                go = false;
            }
            catch (Exception e) {
                e.printStackTrace ();
                crash ("Error in communication");
            }
        }
        System.out.println ("Connection closed cleanly");
    }

    private void crash(String cause) {
        System.out.println ("TCPController: Program aborted");
        System.out.println ("Cause: " + cause);
        System.exit (0);
    }
}