package simplerace;

import javax.swing.*;
import javax.imageio.ImageIO;
import java.awt.*;
import java.io.File;

/**
 * Created by IntelliJ IDEA.
 * User: jtogel
 * Date: 17-Oct-2006
 * Time: 23:55:33
 * To change this template use File | Settings | File Templates.
 */
public class View extends JPanel implements Constants {

    private Vehicle firstVehicle;
    private Vehicle secondVehicle;
    private Track track;
    private final Color[] waypointColors = {Color.BLACK, Color.GRAY, Color.LIGHT_GRAY};
    static Image firstCarImage;
    static Image secondCarImage;
    final int length = 20;
    final int width = 10;
    private JFrame frame;

    static {
        try {
            firstCarImage = ImageIO.read (new File ("../img/redcar.png"));
            secondCarImage = ImageIO.read (new File ("../img/bluecar.png"));
        }
        catch (Exception e) {
            e.printStackTrace ();
        }
    }

    public View (Vehicle firstVehicle, Vehicle secondVehicle, Track track) {
        this.firstVehicle = firstVehicle;
        this.secondVehicle = secondVehicle;
        this.track = track;
    }

    public void paintComponent (Graphics g1) {
        Graphics2D g = (Graphics2D) g1;
        g.setColor (Color.LIGHT_GRAY);
        g.fillRect (0, 0, track.getWidth () + 200, track.getHeight () + 200);
        g.translate (100, 100);
        g.setColor (Color.WHITE);
        g.fillRect (0, 0, track.getWidth (), track.getWidth ());
        Vector2d[] waypoints = track.getAllWaypointsInOrder ();
        for (int i = 0; i < waypoints.length; i++) {
            g.setColor (waypointColors[i]);
            g.fillOval ((int) (waypoints[i].x - waypointDrawingRadius),
                    (int) (waypoints[i].y - waypointDrawingRadius),
                    waypointDrawingRadius * 2, waypointDrawingRadius * 2);
        }
        if (firstVehicle != null) {
            drawVehicle (firstVehicle, g, Color.RED, firstCarImage);
        }
        if (secondVehicle != null) {
            drawVehicle (secondVehicle, g, Color.BLUE, secondCarImage);
        }
        updateTitle ();
    }

    private void drawVehicle (Vehicle vehicle, Graphics2D g, Color chassisColor, Image carImage) {
        // vehicle data
        Vector2d position = vehicle.getPosition ();
        double orientation = vehicle.getOrientation ();
        // position us for painting
        g.translate (position.x, position.y);
        g.rotate (orientation);
        // paint the vehicle
        if (carImage != null) {
            g.drawImage (carImage, -length / 2, -width / 2, null);
        } else {
            // no image. paint a square
            g.setColor (chassisColor);
            g.fillRect (-length / 2, -width / 2, length, width);
            g.setColor (Color.BLACK);
            g.drawLine (0, 0, length / 2, 0);
        }
        // Now, translate back...
        g.rotate (-orientation);
        g.translate (-position.x, -position.y);
    }

    public Dimension getPreferredSize () {
        return new Dimension (track.getWidth () + 200, track.getHeight () + 200);
    }

    public void setFrame (JFrame frame) {
        this.frame = frame;
        updateTitle ();
    }

    private void updateTitle () {
        if (secondVehicle == null) {
            frame.setTitle ("Score: " + track.getFirstFitness ());
        }
        else {
            frame.setTitle ("Score: red " + track.getFirstFitness () +
                    ", blue " + track.getSecondFitness ());
        }
    }

}
